{
  open Global
  open Parser		(* Le type token est dfini dans vhdl.mli *)
  open Vhdl

  exception EndOfInput

  let bit_of_char = function
    | '0' -> O
    | _   -> I
}

let blank = [' ''\t''\n''\r''\014''\000']
let eol = ("\r" | "\r\n" | "\n")

rule token = parse
  (* On saute les blancs et les commentaires *)
  | eol			{ line := !line + 1; token lexbuf }
  | blank		{ token lexbuf }
  | "--"[^'\n']*	{ token lexbuf }

  (* Mots-cls du langage *)
  | "entity"		{ ENTITY }
  | "architecture"	{ ARCHITECTURE }
  | "component"		{ COMPONENT }
  | "port"		{ PORT }
  | "signal"		{ SIGNAL }
  | "is"		{ IS }
  | "of"		{ OF }
  | "begin"		{ BEGIN }
  | "end"		{ END }
  | "in"		{ IN }
  | "out"		{ OUT }
  | "map"		{ MAP }

  (* Ce ne sont pas des mots-cls  proprement parler, mais a simplifie vu qu'on inclut pas
     la bibliothque standard ieee.std_logic_1164 dans laquelle sont dfinis tous ces trucs *)
  | "std_logic"		{ STD_LOGIC }
  | "or"		{ OR }
  | "and"		{ AND }
  | "xor"		{ XOR }
  | "not"		{ NOT }

  (* Dlimiteurs et oprateurs *)
  | '('			{ LPAREN }
  | ')'			{ RPAREN }
  | ','			{ COMMA }
  | ':'			{ COLON }
  | ';'			{ SCOLON }
  | ":="		{ CLEQ }
  | "<="		{ LTEQ }
  | "=>"		{ EQGT }

  (* Constantes et identifieurs *)
  | '\''['0''1']'\''	{ BIT (bit_of_char (Lexing.lexeme lexbuf).[1]) }
  | ['a'-'z''A'-'Z']['a'-'z''A'-'Z''0'-'9''_']*
			{ ID (Lexing.lexeme lexbuf) }

  (* Fin de fichier *)
  | eof			{ raise EndOfInput }

  | _			{ raise (LexerError (!filename, !line, "error near token: '"^(Lexing.lexeme lexbuf)^"'")) }

{
  let input_from_file name =
    let f = open_in name in
    let l = Lexing.from_channel f in
      filename := name;
      Stream.from (fun n -> try Some (Parser.main token l) with EndOfInput -> close_in f; None)

  let input_from_stdin () =
    let l = Lexing.from_channel stdin in
      filename := "<stdin>";
      Stream.from (fun n -> try Some (Parser.main token l) with EndOfInput -> None)
}
