%{
  open Global
  open Vhdl

  let parse_error s =
    raise (ParserError (!filename, !line, s))

%}

%token <Vhdl.bit> BIT
%token <string> ID
%token ENTITY ARCHITECTURE COMPONENT PORT SIGNAL IS OF BEGIN END IN OUT WHEN ELSE MAP
%token STD_LOGIC OR AND XOR NOT
%token LPAREN RPAREN COMMA COLON SCOLON
%token CLEQ LTEQ EQGT EQUAL NOTEQ
%type <Vhdl.t> main

/* priorit des oprateurs */
%left OR AND XOR
%left NOT

/* point d'entre */
%start main

%%

main:
| entity				{ $1 }
| architecture				{ $1 }
;

entity:
| ENTITY ID IS port_list END ID SCOLON	{ if $6 <> $2 then parse_error "mismatching entity names in declaration";
					  Entity { ent_id = $2; ent_iports = $4.comp_iports; ent_oports = $4.comp_oports } }
;

architecture:
| ARCHITECTURE ID OF ID IS declarations BEGIN statements END ID SCOLON
					{ if $10 <> $2 then parse_error "mismatching architecture names in declaration";
					  Architecture { $6 with arch_id = $2; arch_ent = $4; arch_sttms = $8 } }
;

declarations:
| sigdecl declarations			{ { $2 with arch_sgnls = $1::$2.arch_sgnls } }
| component declarations		{ { $2 with arch_comps = $1::$2.arch_comps } }
|					{ { arch_id = ""; arch_ent = ""; arch_sgnls = []; arch_comps = []; arch_sttms = [] } }
;

sigdecl:
| SIGNAL ID COLON STD_LOGIC siginit SCOLON
                                        { { sd_id = $2; sd_init = $5 } }
;

siginit:
| CLEQ BIT                      	{ $2 }
|                                       { U }
;

component:
| COMPONENT ID IS port_list END ID SCOLON
					{ if $6 <> $2 then parse_error "mismatching component names in declaration";
					  { $4 with comp_id = $2 } }
;

statements:
| statement statements			{ $1::$2 }
|					{ [] }
;

statement:
| ID LTEQ expr SCOLON			{ Affectation { aff_dest = $1; aff_expr = $3 } }
| ID COLON ID port_map_list SCOLON	{ Instantiation { inst_id  = $1; inst_ent = $3; inst_maps = $4 } }
;

expr:
| signal				{ Sgnl $1 }
| expr OR expr				{ Op (Or,  [$1;$3]) }
| expr AND expr				{ Op (And, [$1;$3]) }
| expr XOR expr				{ Op (Xor, [$1;$3]) }
| NOT expr				{ Op (Not, [$2])    }
| LPAREN expr RPAREN			{ $2 }
;

signal:
| ID					{ Var $1 }
| BIT					{ Cst $1 }
;

port_list:
| PORT LPAREN ports RPAREN SCOLON	{ $3 }
|					{ { comp_id = ""; comp_iports = []; comp_oports = [] } }
;

ports:
| ID COLON IN  STD_LOGIC SCOLON ports   { { $6 with comp_iports = $1::$6.comp_iports } }
| ID COLON OUT STD_LOGIC SCOLON ports   { { $6 with comp_oports = $1::$6.comp_oports } }
| ID COLON IN  STD_LOGIC		{ { comp_id = ""; comp_iports = [$1]; comp_oports = []   } }
| ID COLON OUT STD_LOGIC		{ { comp_id = ""; comp_iports = [];   comp_oports = [$1] } }
;

port_map_list:
| PORT MAP LPAREN port_map RPAREN	{ $4 }
|					{ [] }
;

port_map:
| ID EQGT signal COMMA port_map		{ { map_port = $1; map_sgnl = $3 }::$5 }
| ID EQGT signal			{ { map_port = $1; map_sgnl = $3 }::[] }
;
