(* Les types ncessaires utiliss par l'AST (l'arbre cr par l'tape de parsing) *)

type id = string

type bit =
  | O
  | I
  | U

type op =
  | Or
  | And
  | Xor
  | Not

type signal =
  | Var of id                                           (* Signal nomm *)
  | Cst of bit                                          (* Constante *)

type expr =
  | Sgnl of signal                                      (* Signal ou constante *)
  | Op   of op * expr list                              (* Opration *)

type sttm_aff = { aff_dest : id;                        (* Signal de destination *)
                  aff_expr : expr }                     (* Expression *)

type sttm_inst_map = { map_port : id;                   (* Nom du port *)
                       map_sgnl : signal }              (* Signal mapp *)

type sttm_inst = { inst_id   : id;                      (* Nom *)
                   inst_ent  : id;                      (* Entit instancie *)
                   inst_maps : sttm_inst_map list }     (* Mapping des ports *)

type statement =
  | Affectation   of sttm_aff                           (* Affectation *)
  | Instantiation of sttm_inst                          (* Instanciation *)

type sigdecl = { sd_id   : id;                          (* Nom *)
                 sd_init : bit }                        (* Valeur initiale *)

type component = { comp_id     : id;                    (* Nom *)
                   comp_iports : id list;               (* Ports d'entre *)
                   comp_oports : id list }              (* Ports de sortie *)

type architecture = { arch_id    : id;                  (* Nom *)
                      arch_ent   : id;                  (* Entit associe *)
                      arch_sgnls : sigdecl list;        (* Signaux dclars *)
                      arch_comps : component list;      (* Composants dclars *)
                      arch_sttms : statement list }     (* Instructions *)

type entity = { ent_id     : id;        		(* Nom *)
                ent_iports : id list;                   (* Ports d'entre *)
                ent_oports : id list }                  (* Ports de sortie *)

type t =
  | Entity       of entity
  | Architecture of architecture

(* Plein de fonctions pour afficher l'AST *)

let string_of_bit = function
  | O -> "0"
  | I -> "1"
  | U -> "U"

let print_component comp tab =
  print_string (tab^"Component: "^comp.comp_id); print_newline ();
  List.iter (fun id -> print_string (tab^"  In:  "^id); print_newline ()) comp.comp_iports;
  List.iter (fun id -> print_string (tab^"  Out: "^id); print_newline ()) comp.comp_oports

let rec print_signal sgnl tab = match sgnl with
  | Var id  -> print_string (tab^"Var: "^id); print_newline ()
  | Cst bit -> print_string (tab^"Cst: "^(string_of_bit bit)); print_newline ()

let rec print_expr expr tab = match expr with
  | Sgnl sgnl      -> print_signal sgnl tab
  | Op (op, exprs) ->
      print_string (tab^"Op: "^(match op with
				  | Or  -> "or"
				  | And -> "and"
				  | Xor -> "xor"
				  | Not -> "not")); print_newline ();
      List.iter (fun expr -> print_expr expr (tab^"  ")) exprs

let print_statement sttm tab = match sttm with
  | Affectation aff ->
      print_string (tab^"Affectation: "^aff.aff_dest); print_newline ();
      print_expr aff.aff_expr (tab^"  ")
  | Instantiation inst ->
      print_string (tab^"Instantiation: "^inst.inst_id^" of "^inst.inst_ent); print_newline ();
      List.iter (fun map ->
		   print_string (tab^"  Map: "^map.map_port); print_newline ();
		   print_signal map.map_sgnl (tab^"    ")) inst.inst_maps

let print obj = match obj with
  | Entity ent ->
      print_string ("Entity: "^ent.ent_id); print_newline ();
      List.iter (fun id -> print_string ("  In:  "^id); print_newline ()) ent.ent_iports;
      List.iter (fun id -> print_string ("  Out: "^id); print_newline ()) ent.ent_oports
  | Architecture arch ->
      print_string ("Architecture: "^arch.arch_ent^"("^arch.arch_id^")"); print_newline ();
      List.iter (fun sgnl -> print_string ("  Signal: "^sgnl.sd_id^" ["^(string_of_bit sgnl.sd_init)^"]"); print_newline ()) arch.arch_sgnls;
      List.iter (fun comp -> print_component comp "  ") arch.arch_comps;
      print_string ("  Statements:"); print_newline ();
      List.iter (fun sttm -> print_statement sttm "    ") arch.arch_sttms
